unit View3D;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, VCLTee.TeEngine, VCLTee.TeeSurfa,
  VCLTee.Chart, TeeChartBlock, VCLTee.TeCanvas, TeeBlocks, Vcl.ExtCtrls,
  VCLTee.TeeProcs, TeeMakerControl, TeeChart3D, Vcl.StdCtrls, VCLTee.TeeScroB,
  VCLTee.TeePoin3, Vcl.ColorGrd, ColorDisplay;

type
  TFView3D = class(TForm)
    HorizontalChartScrollBar: TChartScrollBar;
    VerticalChartScrollBar: TChartScrollBar;
    Chart3D: TChart3D;
    ChartBlock1: TChartBlock;
    CustomBlockChart1: TCustomBlockChart;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure HorizontalChartScrollBarChange(Sender: TObject);
    procedure VerticalChartScrollBarChange(Sender: TObject);
    procedure Chart3DResize(Sender: TObject);
  private
    { Private-Deklarationen }
    FDemo: boolean;
    FSurfaceSeries: TSurfaceSeries;
    FSurfaceWidth: integer;
    FSurfaceDepth: integer;
    FXZPlane: TSurfaceSeries;
    FXZPlanePosition: double;
    FYZPlane: TSurfaceSeries;
    FYZPlanePosition: double;
    FColorDisplay: TColorDisplay;
    function GetGaussDistribution(x, z: integer; Sigma: single): single;
    procedure DrawXZPlane(Position: double);
    procedure DrawYZPlane(Position: double);
    procedure SurfaceSeriesClick(Sender: TChartSeries; ValueIndex: Integer; Button: TMouseButton; Shift: TShiftState; x, y: Integer);
    procedure Chart3DMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
  public
    { Public-Deklarationen }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure ShowDemo;
    procedure SetSurfaceSize(width, Depth: integer);
    procedure Update3DSurface(Data: Pointer);
  end;


implementation

{$R *.dfm}



procedure TFView3D.Chart3DMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X,
  Y: Integer);
begin
messagebeep(64);
end;

procedure TFView3D.Chart3DResize(Sender: TObject);
begin
FColorDisplay.Left:=Chart3D.Width-120;
FColorDisplay.Top:=Chart3D.Top+120;
end;



constructor TFView3D.Create(AOwner: TComponent);
var APalette: TCustom3DPalette;
begin
  inherited Create(AOwner);
  FDemo:=TRUE;
  with Chart3D.Chart do
    begin
      Aspect.Orthogonal:=FALSE;
      Chart3DPercent:=100;
      Aspect.Zoom:=75;
      Legend.Visible:=false;
      Chart3D.OnMouseUp:=Chart3DMouseUp;

      FSurfaceSeries:=TSurfaceSeries(AddSeries(TSurfaceSeries));
      FSurfaceSeries.PaletteStyle:=psRainbow;
      FSurfaceSeries.UseColorRange:= false;
      FSurfaceSeries.UsePalette:= true;
      FSurfaceSeries.IrregularGrid:= true;


      //FSurfaceSeries.WireFrame:=TRUE;   //draw transparent grid

      FXZPlane:=TSurfaceSeries(AddSeries(TSurfaceSeries));
      FXZPlane.IrregularGrid:=true;
//      FXZPlane.Transparency:=50;
      FXZPlane.OnClick:=SurfaceSeriesClick;

      FYZPlane:=TSurfaceSeries(AddSeries(TSurfaceSeries));
      FYZPlane.IrregularGrid:=TRUE;
//      FYZPlane.Transparency:=50;

      FYZPlane.OnClick:=SurfaceSeriesClick;



      FColorDisplay:=TColorDisplay.Create(Chart3D);
      FColorDisplay.Parent:=self;
      FColorDisplay.Left:=Chart3D.Width-120;
      FColorDisplay.Top:=Chart3D.Top+120;
      FColorDisplay.SquareSize:=15;
      FColorDisplay.BorderSize:=1;
      FColorDisplay.AddItem(RGB(8, 0, 0));
      FColorDisplay.AddItem(RGB(16, 0, 0));
      FColorDisplay.AddItem(RGB(24, 0, 0));
      FColorDisplay.AddItem(RGB(32, 0, 0));
      FColorDisplay.AddItem(RGB(40, 0, 0));
      FColorDisplay.AddItem(RGB(48, 0, 0));
      FColorDisplay.AddItem(RGB(56, 0, 0));
      FColorDisplay.AddItem(RGB(64, 0, 0));
      FColorDisplay.AddItem(RGB(72, 0, 0));
      FColorDisplay.AddItem(RGB(80, 0, 0));
      FColorDisplay.AddItem(RGB(88, 0, 0));
      FColorDisplay.AddItem(RGB(96, 0, 0));
      FColorDisplay.AddItem(RGB(104, 0, 0));
      FColorDisplay.AddItem(RGB(112, 0, 0));
      FColorDisplay.AddItem(RGB(120, 0, 0));
      FColorDisplay.AddItem(RGB(128, 0, 0));
      FColorDisplay.AddItem(RGB(136, 0, 0));
      FColorDisplay.AddItem(RGB(144, 0, 0));
      FColorDisplay.AddItem(RGB(152, 0, 0));
      FColorDisplay.AddItem(RGB(160, 0, 0));
      FColorDisplay.AddItem(RGB(168, 0, 0));
      FColorDisplay.AddItem(RGB(176, 0, 0));
      FColorDisplay.AddItem(RGB(184, 0, 0));
      FColorDisplay.AddItem(RGB(192, 0, 0));
      FColorDisplay.AddItem(RGB(200, 0, 0));
      FColorDisplay.AddItem(RGB(208, 0, 0));
      FColorDisplay.AddItem(RGB(216, 0, 0));
      FColorDisplay.AddItem(RGB(224, 0, 0));
      FColorDisplay.AddItem(RGB(232, 0, 0));
      FColorDisplay.AddItem(RGB(240, 0, 0));
      FColorDisplay.AddItem(RGB(248, 0, 0));
      FColorDisplay.AddItem(RGB(255, 0, 0));



      FSurfaceSeries.PaletteStyle:=pscustom;

      FSurfaceSeries.ClearPalette;
      FSurfaceSeries.AddPalette(0, clnavy);
      FSurfaceSeries.AddPalette(0, clblue);
      FSurfaceSeries.AddPalette(0, clteal);
      FSurfaceSeries.AddPalette(0, clgreen);
      FSurfaceSeries.AddPalette(0, clyellow);
      FSurfaceSeries.AddPalette(0, clred);
      FSurfaceSeries.PaletteStep:=768;
      FSurfaceSeries.PaletteSteps:=6;
    end;
end;

destructor TFView3D.Destroy;
begin
  Chart3D.Chart.RemoveAllSeries;
  inherited Destroy;
end;

procedure TFView3D.DrawXZPlane(Position: double);
var x, xs, xe, y: double;
    z: integer;
    PlaneDrawn: boolean;

begin
FXZPlane.BeginUpdate;
FXZPlane.Clear;
xs:=-0.000001*4096+Position;
xe:=position;
x:=xs;

while(x<xe) do
  begin
    z:=0;
    PlaneDrawn:=FALSE;
    while not PlaneDrawn do
      begin
        if(z>=FSurfaceWidth)then
          begin
            z:=FSurfaceWidth-1;
            PlaneDrawn:=TRUE;
          end;
        y:=(Position-x)/0.000001;
        FXZPlane.AddXYZ(x, y, z);

        z:=z+(FSurfaceWidth div 10);
      end;
    x:=x+(xe-xs)/10;
  end;
FXZplane.EndUpdate;
//FXZPlane.Repaint;
end;

procedure TFView3D.DrawYZPlane(Position: double);
var z, zs, ze, y: double;
    x: integer;
    PlaneDrawn: boolean;
begin
FYZPlane.BeginUpdate;
FYZPlane.Clear;
zs:=-0.000001*4096+Position;
ze:=position;
z:=zs;
while(z<ze) do
  begin
    x:=0;
    PlaneDrawn:=FALSE;
    while not PlaneDrawn do
      begin
        if(x>FSurfaceDepth)then
          begin
            x:=FSurfaceDepth-1;
            PlaneDrawn:=TRUE;
          end;
        y:=(Position-z)/0.000001;
        FYZPlane.AddXYZ(x, y, z);
        x:=x+(FSurfaceDepth div 10);
      end;
    z:=z+(ze-zs)/10;
  end;
FYZPlane.EndUpDate;
//FYZPlane.Repaint;

end;

procedure TFView3D.FormClose(Sender: TObject; var Action: TCloseAction);
begin
Action:=caFree;
end;

function TFView3D.GetGaussDistribution(x, z:integer; Sigma: single): single;
begin
 result:=exp(-((x*x+z*z) /(2*(Sigma*Sigma))));
end;

procedure TFView3D.HorizontalChartScrollBarChange(Sender: TObject);
begin
//Chart3D.Aspect.HorizOffset:=-HorizontalChartScrollBar.Position;
DrawXZPlane(HorizontalChartScrollBar.Position);
DrawYZPlane(HorizontalChartScrollBar.Position);
end;



procedure TFView3D.SetSurfaceSize(width, Depth: integer);
var x, z: integer;
    y: double;
begin

FSurfaceWidth:=width;
FSurfaceDepth:=Depth;


y:=0;
for x:= 0 to FSurfaceWidth-1 do
  for z:=0 to FSurfaceDepth-1 do
    begin
     if FDemo then
       y:=4095*GetGaussDistribution(x-128, z-128, 60);
     FSurfaceSeries.AddXYZ(x, y, z);
    end;

DrawXZPlane(100);
DrawYZPlane(100);

FSurfaceSeries.Repaint;
end;

procedure TFView3D.ShowDemo;
begin
FDemo:=TRUE;
SetSurfaceSize(256, 256);
end;

procedure TFView3D.SurfaceSeriesClick(Sender: TChartSeries; ValueIndex: Integer;
  Button: TMouseButton; Shift: TShiftState; x, y: Integer);
begin
//
messagebeep(64);


end;

procedure TFView3D.Update3DSurface(Data: Pointer);
var x, z: integer;
begin
for x:= 0 to width-1 do
  for z:=0 to height-1 do
     FSurfaceSeries.AddXYZ(x, PWord(Integer(Data)+(x*width)+z)^, z);

FSurfaceSeries.Repaint;

end;

procedure TFView3D.VerticalChartScrollBarChange(Sender: TObject);
begin
Chart3D.Aspect.VertOffset:=-VerticalChartScrollBar.Position;
end;

end.
